# frozen_string_literal: true

require 'logging.rb'

module DeepAlgo
  module IL
    module Backend
      class Generic
        def generate(_tree)
          raise 'not yep implemented'
        end

        def document(_tree)
          raise 'not yep implemented'
        end

        def expr(_tree)
          raise 'not yep implemented'
        end

        def add_expr(_tree)
          raise 'not yep implemented'
        end

        def assignment(_tree)
          raise 'not yet implemented'
        end

        def while_expr(_tree)
          raise 'not yep implemented'
        end

        def while_cond_expr(_tree)
          raise 'not yep implemented'
        end

        def plus_expr(_tree)
          raise 'not yep implemented'
        end

        def minus_expr(_tree)
          raise 'not yep implemented'
        end

        def time_expr(_tree)
          raise 'not yep implemented'
        end

        def div_expr(_tree)
          raise 'not yep implemented'
        end

        def dotted_expr(_tree)
          raise 'not yep implemented'
        end

        def factor(_tree)
          raise 'not yep implemented'
        end

        def tab_ref(_tree)
          raise 'not yep implemented'
        end

        def terminal(_tree)
          raise 'not yep implemented'
        end

        def dotted_terminal(_tree)
          raise 'not yep implemented'
        end

        def atom(_tree)
          raise 'not yep implemented'
        end

        def descriptor(_tree)
          raise 'not yep implemented'
        end

        def descriptor_items(_tree)
          raise 'not yep implemented'
        end

        def descriptor_item(_tree)
          raise 'not yep implemented'
        end

        def call(_tree)
          raise 'not yep implemented'
        end

        def args(_tree)
          raise 'not yep implemented'
        end

        def parenthesed(_tree)
          raise 'not yep implemented'
        end

        def colored(_tree)
          raise 'not yep implemented'
        end

        def color(_tree)
          raise 'not yep implemented'
        end

        def colorcode(_tree)
          raise 'not yep implemented'
        end

        def identifier(_tree)
          raise 'not yep implemented'
        end

        def number(_tree)
          raise 'not yep implemented'
        end

        def unary_minus(_tree)
          'not yep implemented'
        end

        def text(_tree)
          raise 'not yep implemented'
        end
      end

      class Latex < Generic
        def initialize
          @curr_backend = nil
        end

        def generate(tree, curr_backend)
          @curr_backend = curr_backend
          latex = tree.value(@curr_backend)
          latex
        end

        def document(tree)
          tree.elements[0].value(@curr_backend)
        end

        def add_expr(tree)
          tree.elements[0].value(@curr_backend)
        end

        def assignment(tree)
          left = tree.elements[0].value(@curr_backend)
          right = tree.elements[4].value(@curr_backend)
          "#{left} \\text{equals} #{right}"
        end

        def plus_expr(tree)
          left = tree.elements[0].value(@curr_backend)
          right = tree.elements[4].value(@curr_backend)
          "#{left} + #{right}"
        end

        def minus_expr(tree)
          left = tree.elements[0].value(@curr_backend)
          right = tree.elements[4].value(@curr_backend)
          "#{left} - #{right}"
        end

        def time_expr(tree)
          left = tree.elements[0].value(@curr_backend)
          right = tree.elements[4].value(@curr_backend)
          "#{left} \\times #{right}"
        end

        def parenthesed(tree)
          "\\left( #{tree.elements[2].value(@curr_backend)} \\right)"
        end

        def colored(tree)
          color = "##{tree.elements[0].value(@curr_backend)}"
          expr = tree.elements[4].value(@curr_backend)
          "\\color{ #{color} }{ #{expr} }"
        end

        def color(tree)
          tree.elements[1].value(@curr_backend)
          # tree.text_value
        end

        def colorcode(tree)
          tree.text_value
        end

        def atom(tree)
          tree.text_value
        end

        def while_expr(tree)
          statements = tree.elements[4].value(@curr_backend)
          cond = tree.elements[8].value(@curr_backend)
          "\\text{repeat} \\left[ #{statements} \\right]#{cond}"
        end

        def while_cond_expr(tree)
          "_{ \\text{ while #{tree.elements[4].value(@curr_backend)} } }"
        end

        def text(tree)
          tree.text_value
        end

        def dotted_expr(tree)
          to_return = ''
          if !tree.elements[1].nil? && tree.elements[1] != ''
            tree
              .elements[1]
              .elements
              .each do |son_expr|
              to_return = "#{to_return}. \
              #{son_expr.elements[2].value(@curr_backend)}"
            end
          end

          "#{tree.elements[0].value(@curr_backend)}#{to_return}"
        end

        def call(tree)
          caller_name = tree.elements[0].value(@curr_backend)
          params = tree.elements[4].value(@curr_backend)
          "#{caller_name} \\left( #{params} \\right)"
        end

        def dotted_terminal(tree)
          first_expr = tree.elements[0].value(@curr_backend)
          second_expr = tree.elements[2].value(@curr_backend)
          "#{first_expr}.#{second_expr}"
        end

        def args(tree)
          # parse arg, arg, arg, ...
          to_return = ''
          tree
            .elements[1]
            &.elements
            &.each do |arg|
            to_return = "#{to_return}, \
            #{arg.elements[3].value(@curr_backend)}"
          end
          "#{tree.elements[0].value(@curr_backend)}#{to_return}"
        end

        def tab_ref(tree)
          tab = tree.elements[0].value(@curr_backend)
          index = tree.elements[6].value(@curr_backend)
          "#{tab}_{ #{index} }"
        end

        def descriptor(tree)
          label = tree.elements[2].value(@curr_backend)
          items = tree.elements[8].value(@curr_backend)
          "\\text{ #{label} } \
           \\left\\{\\begin{array}{l} #{items} \\\\ \\end{array}\\right."
        end

        def descriptor_items(tree)
          to_return = ''
          tree
            .elements[1]
            &.elements
            &.each do |arg|
            to_return = "#{to_return} \\\\ \
             #{arg.elements[1].value(@curr_backend)} "
          end
          "#{tree.elements[0].value(@curr_backend)} #{to_return}"
        end

        def descriptor_item(tree)
          label = tree.elements[4].value(@curr_backend)
          assignee = tree
                     .elements[10]
                     .elements[0]
                     .value(@curr_backend)
          "\\text{ #{label} } = #{assignee}"
        end

        def div_expr(tree)
          numerator = tree.elements[6].value(@curr_backend)
          denominator = tree.elements[14].value(@curr_backend)
          "\\dfrac{ #{numerator} }{ #{denominator} }"
        end

        def unary_minus(tree)
          "-#{tree.elements[2].value(@curr_backend)}"
        end

        def identifier(tree)
          tree.text_value
        end
      end
    end
  end
end
